# -*- coding: utf-8 -*-
from Preset.Model.PartBase import PartBase
from Preset.Model.GameObject import registerGenericClass


@registerGenericClass("APart")
class APart(PartBase):
	def __init__(self):
		super(APart, self).__init__()
		# 零件名称(之前步骤中将名称改为了A)
		self.name = "A"
		# 帧数变量用于之后逻辑判断
		self.tickIndex = 0

	def printSelfMsg(self, msg):
		# 区分时客户端零件打印还是服务端的零件打印
		print("client:" if self.isClient else "server:", msg)

	# 逻辑处理函数
	def OnHandleClientEvent(self, msg):
		if "msg" in msg:
			# 打印接收到的消息
			self.printSelfMsg(msg["msg"])
		if "id" in msg:
			# 处理逻辑
			player = msg["id"]
			# 获取发送信息玩家的当前位置
			pos = list(self.GetEntityFootPos(player))
			pos[1] += 50
			# 设置该玩家的位置
			self.SetEntityFootPos(player, tuple(pos))
			# 在发送ClientEvent事件的玩家客户端上显示红色的文字
			self.NotifyOneMessage(player, "起飞!", "§c")
			# 接口使用效果展示，在客户端发送LogicEvent事件给指定的客户端
			self.NotifyToClient(player, "LogicEvent", "from player {} part {}_OnHandleClientEvent_NotifyToClient_{}".format(self.GetLocalPlayerId(), self.name, self.tickIndex))

	def InitClient(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 客户端的零件对象初始化入口
		@description 客户端的零件对象初始化入口
		"""
		# 监听自身内部的MsgEvent与LogicEvent事件
		self.ListenSelfEvent("MsgEvent", self, self.printSelfMsg)
		self.ListenSelfEvent("LogicEvent", self, self.printSelfMsg)

	def InitServer(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 服务端的零件对象初始化入口
		@description 服务端的零件对象初始化入口
		"""
		# 监听自身内部的MsgEvent与LogicEvent事件
		self.ListenSelfEvent("MsgEvent", self, self.printSelfMsg)
		self.ListenSelfEvent("LogicEvent", self, self.OnHandleClientEvent)

	def TickClient(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 客户端的零件对象逻辑驱动入口
		@description 客户端的零件对象逻辑驱动入口
		"""
		self.tickIndex += 1
		if self.tickIndex % 600 == 0:
			# 广播MsgEvent事件，用于验证其余另外两个Notify接口不一样的地方
			self.BroadcastEvent("MsgEvent", "tickClient_BroadcastEvent_Msg")
			# 在客户端发送LogicEvent事件给服务端，并且发送本地玩家的id与对应消息
			self.NotifyToServer("LogicEvent", {
				"id": self.GetLocalPlayerId(),
				"msg": "from player {} part {}_TickClient_NotifyToServer_{}".format(self.GetLocalPlayerId(), self.name, self.tickIndex)
			})

	def TickServer(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 服务端的零件对象逻辑驱动入口
		@description 服务端的零件对象逻辑驱动入口
		"""
		self.tickIndex += 1
		if self.tickIndex % 600 == 60:
			# 取消到第60帧那一次，保持和客户端的频率同步
			if self.tickIndex > 60:
				# 广播MsgEvent事件，用于验证其余另外两个接口不一样的地方
				self.BroadcastEvent("MsgEvent", "TickServer_BroadcastEvent_Msg")
				self.BroadcastToAllClient("MsgEvent", "from player {} part {}_TickServer_BroadcastToAllClient_Msg".format(self.GetLocalPlayerId(), self.name))

	def DestroyClient(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 客户端的零件对象销毁逻辑入口
		@description 客户端的零件对象销毁逻辑入口
		"""
		# 反监听事件,确保安全退出
		self.UnListenSelfEvent("LogicEvent", self, self.printSelfMsg)
		self.UnListenSelfEvent("MsgEvent", self, self.printSelfMsg)

	def DestroyServer(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 服务端的零件对象销毁逻辑入口
		@description 服务端的零件对象销毁逻辑入口
		"""
		# 反监听事件,确保安全退出
		self.UnListenSelfEvent("LogicEvent", self, self.OnHandleClientEvent)
		self.UnListenSelfEvent("MsgEvent", self, self.printSelfMsg)
