# -*- coding: utf-8 -*-
from Preset.Model.PartBase import PartBase
from Preset.Model.GameObject import registerGenericClass


@registerGenericClass("DPart")
class DPart(PartBase):
	def __init__(self):
		super(DPart, self).__init__()
		# 零件名称
		self.name = "D"
		# 帧数变量用于之后逻辑判断
		self.tickIndex = 0

	def InitClient(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 客户端的零件对象初始化入口
		@description 客户端的零件对象初始化入口
		"""
		pass

	def printSelfMsg(self, msg):
		# 区分时客户端零件打印还是服务端的零件打印
		print("client:" if self.isClient else "server:", msg)

	def InitServer(self):
		import Preset.Controller.PresetApi as presetApi
		# 在预设系统下获取指定类型(BPart)与指定名称(B)的零件（不用在同一个预设下）
		partB = presetApi.GetGameObjectByTypeName("BPart", "B")

		if partB:
			# 监听来自于的partB事件
			print ("partD listen PartEventB")
			self.ListenPartEvent(partB.id, "PartEventB", self, self.printSelfMsg)

		# 获取父预设的一个名称为(C)的子零件
		partC = self.GetParent().GetPartByName("C")

		if partC:
			# 监听来自于的partC事件
			print ("partD listen PartEventC")
			self.ListenPartEvent(partC.id, "PartEventC", self, self.printSelfMsg)

		# 监听预设系统的事件
		self.ListenPresetSystemEvent("PresetSystemEvent", self, self.printSelfMsg)

	def TickClient(self):
		pass

	def TickServer(self):
		self.tickIndex += 1
		if self.tickIndex % 600 == 60:
			# 用作对比
			self.BroadcastEvent("PartEventB", "from player {} part {}_TickServer_{}".format(self.GetLocalPlayerId(), self.name, self.tickIndex))
			self.BroadcastEvent("PartEventC", "from player {} part {}_TickServer_{}".format(self.GetLocalPlayerId(), self.name, self.tickIndex))
			self.BroadcastEvent("PresetSystemEvent", "from player {} part {}_TickServer_{}".format(self.GetLocalPlayerId(), self.name, self.tickIndex))

	def DestroyClient(self):
		"""
		@classify_path 预设对象/零件/零件PartBase
		@state 1.23.0 新增 客户端的零件对象销毁逻辑入口
		@description 客户端的零件对象销毁逻辑入口
		"""
		pass

	def DestroyServer(self):
		import Preset.Controller.PresetApi as presetApi
		partB = presetApi.GetGameObjectByTypeName("BPart", "B")
		if partB:
			self.UnListenPartEvent(partB.id, "PartEventB", self, self.printSelfMsg)
		partC = self.GetParent().GetPartByName("C")
		if partC:
			self.UnListenPartEvent(partC.id, "PartEventC", self, self.printSelfMsg)
		self.UnListenPresetSystemEvent("PresetSystemEvent", self, self.printSelfMsg)
